SystemOrganization addCategory: #'CT-Monticello-Tests'!
SystemOrganization addCategory: #'CT-Monticello-Core'!

MCMcdReader subclass: #CtMcdReader
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CT-Monticello-Core'!

!CtMcdReader class methodsFor: 'testing' stamp: 'mt 9/29/2010 12:45:39.97'!
canReadFileNamed: fileName
	"Replacement for MCMcdReader."

	^ (fileName endsWith: self extension)! !

!CtMcdReader methodsFor: 'accessing' stamp: 'mt 9/29/2010 13:17:03.838'!
basicVersion

	^ CtMcDiffyVersion
		package: self package
		info: self info
		dependencies: self dependencies
		baseInfo: self baseInfo
		patch: self patch! !

MCVersion subclass: #CtMcVersion
	instanceVariableNames: 'markups'
	classVariableNames: 'PostLoadMergeActions'
	poolDictionaries: ''
	category: 'CT-Monticello-Core'!

!CtMcVersion class methodsFor: 'post load/merge actions' stamp: 'mt 9/29/2010 14:35:45.375'!
addPostLoadMergeAction: aBlockWithOneArgument
	"Allows you to register actions from the outside that do something based on a version object that is given into the block."

	(PostLoadMergeActions ifNil: [PostLoadMergeActions := OrderedCollection new])
		add: aBlockWithOneArgument.
		
	"Example: 
	[ :version | (Preferences
		preferenceAt: #browseMarkupsAfterLoadMerge
		ifAbsent: [Preference new rawValue: false])
		preferenceValue ifTrue: [ CtMarkupBrowser
			openOn: (OBClassCategoryNode on: version package name)] ]."! !

!CtMcVersion class methodsFor: 'post load/merge actions' stamp: 'mt 9/29/2010 14:32:32.765'!
clearPostLoadMergeActions

	PostLoadMergeActions := OrderedCollection new.! !

!CtMcVersion class methodsFor: 'instance creation' stamp: 'mt 9/29/2010 13:14:23.468'!
package: aPackage info: aVersionInfo snapshot: aSnapshot dependencies: aCollection markups: aMarkupSnapshot

	^ (self
		package: aPackage
		info: aVersionInfo
		snapshot: aSnapshot
		dependencies: aCollection)
		markups: aMarkupSnapshot! !

!CtMcVersion methodsFor: 'actions' stamp: 'mt 9/29/2010 15:28:24.36'!
load

	super load.
	[SystemMarkups loadSnapshot: self markups.]
		on: Error
		do: [self inform: 'The markup snapshot was not loaded.'].
	
	(PostLoadMergeActions ifNil: [PostLoadMergeActions := OrderedCollection new])
		do: [:action | action value: self].! !

!CtMcVersion methodsFor: 'accessing' stamp: 'mt 9/29/2010 13:23:18.447'!
markups

	^ markups! !

!CtMcVersion methodsFor: 'accessing' stamp: 'mt 9/29/2010 13:25:16.407'!
markups: aMarkupSnapshot

	markups ifNotNil: [Error signal: 'Markup snapshot can only be initialized once!!'].
	markups := aMarkupSnapshot.! !

!CtMcVersion methodsFor: 'actions' stamp: 'mt 9/29/2010 15:28:47.857'!
merge

	super merge.
	[SystemMarkups loadSnapshot: self markups.]
		on: Error
		do: [self inform: 'The markup snapshot was not loaded.'].
	
	(PostLoadMergeActions ifNil: [PostLoadMergeActions := OrderedCollection new])
		do: [:action | action value: self].! !

!CtMcVersion methodsFor: 'accessing' stamp: 'mt 9/29/2010 13:26:38.351'!
writerClass

	^ CtMczWriter! !

!MCVersion methodsFor: '*ct-monticello-override' stamp: 'mt 9/29/2010 13:35:48.905'!
writerClass
	"This override is needed for newly created versions. This avoids an override in the MCWorkingCopy class."
	
	^ CtMczWriter ! !

MCMczWriter subclass: #CtMczWriter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CT-Monticello-Core'!

!CtMczWriter class methodsFor: 'accessing' stamp: 'mt 9/29/2010 11:58:33.131'!
readerClass

	^ CtMczReader! !

!CtMczWriter methodsFor: 'visiting' stamp: 'mt 9/29/2010 13:22:36.231'!
writeMarkups: aPackageName

	self
		addString: (self serializeInBinary: (SystemMarkups createSnapshot: aPackageName))
		at: 'markups.bin'.! !

!CtMczWriter methodsFor: 'visiting' stamp: 'mt 9/29/2010 13:22:19.19'!
writeVersion: aVersion
	"Override to write markups."

	SwaSourceTextStyler instance disable.

	self writeFormat.
	self writePackage: aVersion package.
	self writeVersionInfo: aVersion info.
	self writeDefinitions: aVersion.
	
	"No need to work with a custom version with markups. Just use the package name."
	self writeMarkups: aVersion package name.
	
	aVersion dependencies do: [:ea | self writeVersionDependency: ea].
	
	SwaSourceTextStyler instance enable.! !

MCMczReader subclass: #CtMczReader
	instanceVariableNames: 'markups'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CT-Monticello-Core'!

!CtMczReader class methodsFor: 'testing' stamp: 'mt 9/29/2010 12:43:58.25'!
canReadFileNamed: fileName
	"Replacement for MCMczReader."

	^ (fileName endsWith: self extension)! !

!CtMczReader methodsFor: 'accessing' stamp: 'mt 9/29/2010 13:14:58.46'!
basicVersion
	"Reimplemented to store the markup snapshot into the version object."
	
	^ CtMcVersion
		package: self package
		info: self info
		snapshot: self snapshot
		dependencies: self dependencies
		markups: self markups! !

!CtMczReader methodsFor: 'loading' stamp: 'mt 9/29/2010 11:59:58.321'!
loadMarkups

	(self zip memberNamed: 'markups.bin') ifNotNilDo:
		[:m | [markups := (DataStream on: m contentStream) next]
			on: Error do: [:err | self inform: 'Could not load snapshot of markups!!']].! !

!CtMczReader methodsFor: 'accessing' stamp: 'mt 9/29/2010 11:59:47.319'!
markups

	markups ifNil: [self loadMarkups].
	^ markups! !

!MCMczReader class methodsFor: '*ct-monticello' stamp: 'mt 9/29/2010 12:43:30.227'!
canReadFileNamed: fileName
	"Overridden to use custom MCZ reader."

	^ false! !

MCMcdWriter subclass: #CtMcdWriter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CT-Monticello-Core'!

!CtMcdWriter class methodsFor: 'accessing' stamp: 'mt 9/29/2010 13:04:15.131'!
readerClass

	^ CtMcdReader! !

TestCase subclass: #CtMonticelloTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CT-Monticello-Tests'!

!CtMonticelloTest methodsFor: 'running' stamp: 'mt 9/29/2010 14:03:30.567'!
setUp

	super setUp.
	! !

!CtMonticelloTest methodsFor: 'tests' stamp: 'mt 9/29/2010 12:59:34.144'!
test01mczReaderClass

	self
		assert: CtMczReader
		equals: (MCVersionReader readerClassForFileNamed: 'CT-Monticello-mt.2.mcz').! !

!CtMonticelloTest methodsFor: 'tests' stamp: 'mt 9/29/2010 12:59:41.436'!
test02mcdReaderClass

	self
		assert: CtMcdReader
		equals: (MCVersionReader readerClassForFileNamed: 'CT-Monticello-mt.2.mcd').! !

!CtMonticelloTest methodsFor: 'tests' stamp: 'mt 9/29/2010 12:59:48.325'!
test03versionWriterClass

	self
		assert: CtMczWriter
		equals: MCVersion new writerClass.! !

!CtMonticelloTest methodsFor: 'tests' stamp: 'mt 9/29/2010 12:59:55.139'!
test04diffyVersionWriterClass

	self
		assert: CtMcdWriter
		equals: MCDiffyVersion new writerClass.! !

!CtMonticelloTest methodsFor: 'tests' stamp: 'mt 9/29/2010 13:00:02.856'!
test05readerClassForMczWriter

	self
		assert: CtMczReader
		equals: CtMczWriter readerClass.! !

!CtMonticelloTest methodsFor: 'tests' stamp: 'mt 9/29/2010 13:00:09.867'!
test06readerClassForMcdWriter

	self
		assert: CtMcdReader
		equals: CtMcdWriter readerClass.! !

!CtMonticelloTest methodsFor: 'tests' stamp: 'mt 9/29/2010 14:06:39.389'!
test07writeNewVersion
	"MCWorkingCopy creates an MCVersion instance. Writing should work too for this."
	
	| version stream mczFileContents reader |
	version := self version.
	stream := RWBinaryOrTextStream on: (String new: 100).
	version fileOutOn: stream.
	mczFileContents := stream contents.
	reader := (MCVersionReader readerClassForFileNamed: 'Dummy-mt.1.mcz') on: mczFileContents readStream.
	
	self
		assert: (reader markups isKindOf: CtMarkupDatabaseSnapshot);
		assert: version package name equals: reader markups packageName;
		assert: reader markups activeMarkups isEmpty;
		assert: reader markups equals: reader version markups.! !

!CtMonticelloTest methodsFor: 'tests' stamp: 'mt 9/29/2010 14:21:11.782'!
test08writeNewVersionWithMarkups
	
	| version markup stream mczFileContents reader |
	"Step 1 - Create a markup."
	markup := CtMarkupGenerator new
		addTag: #talk;
		addTarget: (CtMethodSourceTarget
			class: self class
			selector: thisContext selector
			from: 1
			to: 10);
		next.
	SystemMarkups addMarkup: markup.
	
	"Step 2 - Try so put that markup into MCZ file."
	version := self version.
	stream := RWBinaryOrTextStream on: (String new: 100).
	version fileOutOn: stream.
	mczFileContents := stream contents.
	reader := (MCVersionReader readerClassForFileNamed: 'Dummy-mt.1.mcz') on: mczFileContents readStream.
	
	self
		assert: markup equals: (reader markups
			markupsForClass: self class
			selector: thisContext selector
			kind: #methodSource) first.
			
	SystemMarkups removeMarkup: markup.! !

!CtMonticelloTest methodsFor: 'tests' stamp: 'mt 9/29/2010 14:41:15.179'!
test09postLoadMergeActions
	
	| version stream mczFileContents reader value |
	version := self versionDummy.
	stream := RWBinaryOrTextStream on: (String new: 100).
	version fileOutOn: stream.
	mczFileContents := stream contents.
	reader := (MCVersionReader readerClassForFileNamed: 'Dummy-mt.1.mcz') on: mczFileContents readStream.
	
	CtMcVersion addPostLoadMergeAction: [:ver | value := ver].
	reader version load.
	self assert: value markups isEmpty.
	CtMcVersion clearPostLoadMergeActions.
	
	CtMcVersion addPostLoadMergeAction: [:ver | value := #foobar].
	reader version merge.
	self assert: #foobar equals: value.
			
	CtMcVersion clearPostLoadMergeActions.! !

!CtMonticelloTest methodsFor: 'support' stamp: 'mt 9/29/2010 14:06:18.103'!
version

	^ MCVersion
		package: (MCPackage named: #'CT-Monticello')
		info: (MCVersionInfo
			name: #TestVersion
			id: UUID new
			message: 'This is just a test.'
			date: nil
			time: nil
			author: 'mt'
			ancestors: #())! !

!CtMonticelloTest methodsFor: 'support' stamp: 'mt 9/29/2010 14:37:08.734'!
versionDummy

	^ MCVersion
		package: (MCPackage named: #Foobar)
		info: (MCVersionInfo
			name: #TestVersion
			id: UUID new
			message: 'This is just a test.'
			date: nil
			time: nil
			author: 'mt'
			ancestors: #())! !

MCDiffyVersion subclass: #CtMcDiffyVersion
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CT-Monticello-Core'!

!CtMcDiffyVersion methodsFor: 'accessing' stamp: 'mt 9/29/2010 13:26:54.71'!
writerClass

	^ CtMcdWriter! !

!MCDiffyVersion methodsFor: '*ct-monticello-override' stamp: 'mt 9/29/2010 13:35:02.687'!
writerClass

	^ CtMcdWriter! !
